#include <cstdlib>
#include <cstdio>
#include <utility>

#include "../coco/coco.h"
#include "population.h"
#include "combination.h"


void simple_combination(Population & pop_in,
                        Population & pop_out,
                        const double *lower_bounds,
                        const double *upper_bounds,
                        int parent1,
                        int parent2,
                        const double eta,
                        coco_random_state_t *random_generator)
{
  double dimension = pop_in.getDim();

  for (int j=0; j<dimension; j++){
    double u = coco_random_uniform(random_generator);
    double beta;

    if (u <= 0.5){
      beta = pow(2*u,1/(1+eta));
    }
    else {
      beta = pow(1/(2*(1-u)),1/(1+eta));
    }
    double child1 = 0.5*((1+beta)*pop_in.getIndiv(parent1,j) + (1-beta)*pop_in.getIndiv(parent2,j));
    double child2 = 0.5*((1-beta)*pop_in.getIndiv(parent1,j) + (1+beta)*pop_in.getIndiv(parent2,j));

    if (child1 > upper_bounds[j]){
      child1 = upper_bounds[j];
    }
    if (child1 < lower_bounds[j]){
      child1 = lower_bounds[j];
    }
    if (child2 > upper_bounds[j]){
      child2 = upper_bounds[j];
    }
    if (child2 < lower_bounds[j]){
      child2 = lower_bounds[j];
    }
    pop_out.setIndiv(parent1,j,child1);
    pop_out.setIndiv(parent2,j,child2);
  }
}

/* Recombinaison prédéfini */
void combinate(Population & pop_in,
               Population & pop_out,
               const double *lower_bounds,
               const double *upper_bounds,
               double eta,
               double p_combination,
               coco_random_state_t *random_generator)
{
  double u;
  double size = pop_in.getCurrentSize();
  u = coco_random_uniform(random_generator);
  for (int i=0; i<floor(size/2.0); i++){
      if (u <= p_combination){
        simple_combination(pop_in,pop_out,lower_bounds,upper_bounds,i,size-i-1, eta, random_generator);
      }
    }
  pop_out.setCurrenSize(size);
}
