#include <cmath>
#include <cstdio>

#include "../coco/coco.h"
#include "population.h"
#include "tournament.h"
#include "combination.h"
#include "mutation.h"
#include "ibea.h"

void ibea(evaluate_function_t evaluate,
          const size_t dimension,
          const size_t number_of_objectives,
          const double *lower_bounds,
          const double *upper_bounds,
          const size_t max_budget,
          coco_random_state_t *random_generator)
{
  /* Parameters */
  int n_individus = 100; /* number of individus per generations */
  int n_generations = floor(max_budget/n_individus); /* number of generations */
  int mu = 20; /* number of binary tournament */
  double eta = 20; /* polynomiale mutation degree */
  double p_combination = 1;
  double p_mutation = 0.01;
  double kappa = 0.05;

  /* Initialize */
  Population pop = Population(n_individus, dimension, number_of_objectives);
  Population new_pop = Population(n_individus, dimension, number_of_objectives);
  pop.random_init(lower_bounds, upper_bounds, random_generator);

  /* Iterations */
  int m = 0;
  while (m < n_generations){
    /* 2 Fitness assignment */
    pop.evaluate(evaluate);
    pop.compute_indicators();
    pop.compute_fitnesses(kappa);

    new_pop.empty();
    while (new_pop.getCurrentSize() < n_individus){
      /* 3 Environemental select */
      pop.darwin(kappa);

      /* 4 */

      /* 5 Fill mating pool */
      binary_tournament(pop, new_pop, mu, random_generator);
    }

    /* 6 Combination and Mutation */
    pop.empty();
    combinate(new_pop, pop, lower_bounds, upper_bounds, eta, p_combination,random_generator);
    mutate(pop, lower_bounds, upper_bounds, eta, p_mutation, random_generator);
    m++;
  }
}
