#include "indicator.h"

/**
* Returns the associated espsilon-indicator. In the complete algorithm, we only
* need to take as arguments an only vector, but we still use here A and B as
* sets of vectors. Furthermore, instead of calling in every
* epsilon_indicator the method coco_evaluate, we better save every f(x) and
* enter as parameters of the function the two vectors f(x1) and f(x2) we want
* to compare. Each indicator uses normalized terms :
*   f_i'(x)=(f_i(x)-bmin[i])/(bmax[i]-bmin[i])
* @param x: image of the first vector to compare by f
* @param y: image of the second vector to compare by f
* @param bmin: vector containing minimum bounds of the f(x) on the population
* @param bmin: vector containing maximum bounds of the f(x) on the population
* @returns: the espsilon-indicator
*/
double epsilon_indicator(double * x, double * y, double * bmin, double * bmax, int dim_obj) {
  int k;
  double eps_k=0.0, eps_temp;

  /* Here we search for the value of
  * I (A, B) =
  *   min{eps} (∀x2 ∈ B ∃x1 ∈ A : fi(x1) − eps ≤ fi(x2) for i ∈ {1,...,n}).
  * eps is between 0 and the dimension of the vectors
  *
  */

  for (k = 0; k < dim_obj; k++) {
    eps_temp = (y[k] - x[k]) / (bmax[k] - bmin[k]);
    if (k == 0)
      eps_k = eps_temp;
    else {
      if (eps_k < eps_temp){
        eps_k = eps_temp;
      }
    }
  }

  return eps_k;
}
