#include <cstdlib>
#include <cstdio>
#include <cmath>
#include <algorithm>

#include "../coco/coco.h"
#include "population.h"
#include "mutation.h"

double polynomiale_mutation(double parent,
                            const double lower_bound,
                            const double upper_bound,
                            double eta,
                            coco_random_state_t *random_generator)
{
  double child;
  double u = coco_random_uniform(random_generator);
  double delta = std::min((parent - lower_bound)/(upper_bound - lower_bound),
                          (upper_bound - parent)/(upper_bound - lower_bound));
  double delta_q;

  if (u <= 0.5){
    delta_q = pow(pow((2*u) + (1-2*u)*(1-delta), (eta+1)), (1/(1+eta))) - 1;
  }
  else {
    delta_q = 1 - pow((2*(1-u) + 2*(u-0.5)*pow((1-delta), (eta+1))), (1/(1+eta)));
  }

  child = parent + delta_q*(upper_bound - lower_bound);
  if (child > upper_bound){
    child = upper_bound;
  }
  if (child < lower_bound){
    child = lower_bound;
  }

  return child;
}

void mutate(Population & population,
            const double *lower_bounds,
            const double *upper_bounds,
            double eta,
            double p_mutation,
            coco_random_state_t *random_generator)
{
  double u;
  double size = population.getSize();
  double dimension = population.getDim();

  for (int i=0; i<size; i++){
    for (int j=0; j<dimension; j++){
      u = coco_random_uniform(random_generator);
      if (u <= p_mutation){
        population.setIndiv(i,j,polynomiale_mutation(population.getIndiv(i,j), lower_bounds[j], upper_bounds[j], eta, random_generator));
      }
    }
  }

}
