#ifndef POPULATION_H
#define POPULATION_H

typedef void (*evaluate_function_t)(const double *x, double *y);

class Population {
public:
  Population(int size_max, int dimension, int n_objectives);
  ~Population();
  int getSize();
  int getCurrentSize();
  void setCurrenSize(int n);
  int getDim();

  void empty();

  void setIndiv(int i, int j, double value);
  double getIndiv(int i, int j);
  void setFit(int i, double value);
  double getFit(int i);
  void setMin(int i);
  int getMin();

  void random_init(const double *lower_bounds,
                   const double *upper_bounds,
                   coco_random_state_t *random_generator);
  void evaluate(evaluate_function_t evaluate_function);
  void compute_indicators();
  void compute_fitnesses(double kappa);
  void darwin(double kappa);

  void printFitnesses();
  void printIndicatorValues();
  void printValues();
  void printPopulation();

private:
  int size_;
  int current_size_;
  int dimension_;
  int n_objectives_;
  double ** population_;
  double ** values_;
  double ** indicator_values_;
  double * fitnesses_;
  double * bmin_;
  double * bmax_;

  int ind_min_;
  double c_;
};

#endif
